// ===================================================================
// (C) Copyright IBM Corp. 2005, 2006 All Rights Reserved.
//                                                                           
// Licensed Materials - Property of IBM
//                                                          
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//              
// You may copy, modify, and distribute this sample program in any
// form without payment to IBM, for any purpose including developing,
// using, marketing or distributing programs that include or are
// derivative works of the sample program.
//                                                                           
// The sample program is provided to you on an "AS IS" basis, without
// warranty of any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES,
// EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Some jurisdictions do not allow for the exclusion or limitation of
// implied warranties, so the above limitations or exclusions may not
// apply to you. IBM shall not be liable for any damages you suffer as
// a result of using, modifying or distributing the sample program or
// its derivatives.
//===================================================================
package com.ibm.wbimonitor.samples.db2emitter;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.ibm.wbimonitor.samples.emitterframework.EventFormatter;
import com.ibm.wbimonitor.samples.emitterframework.EventFormatterFactory;
import com.ibm.wbimonitor.samples.emitterframework.exception.EventFormatterException;
import com.ibm.wbimonitor.samples.emitterframework.exception.EventFormatterNotFoundException;

/*
 * This class implements the EventFormatterFactory interface.
 */
public class DB2EventFormatterFactory implements EventFormatterFactory {

    private static Logger logger = Logger.getLogger( "DB2EventFormatterFactory" );

    /*
     * The name of the file defining the event type-to-formatter mapping.
     */
    private static String FORMATTERFACTORY_PROPERTIES = "/DB2EventFormatter.properties";

    /*
     * Synchronization object.
     */
    private static Object lock = new Object( );

    /*
     * Properties object into which to load the mapping definitions.
     */
    private static Properties properties = null;

    /*
     * Returns an EventFormatter suitable for the events of the type specified
     * in the EventTableBean object. This EventFormatter knows the specifics of
     * how to convert the specified event type into a well-formatted CBE
     * expected by WB Monitor. The selected and returned EventFormatter is based
     * on the static mapping defined in the DB2EventFormatter.properties file.
     */
    public EventFormatter getEventFormatter( Object o ) throws EventFormatterException {

        initializeProperties( );

        String type = ( (EventTableBean)o ).getType( );
        if( type == null ) {
            logger.log( Level.INFO, "(type == null)" );
            throw new EventFormatterException( "The event type was not specified in the EventFileBean.  Unable to obtain an event formatter." );
        }
        String eventFormatterClassName = properties.getProperty( type );
        if( eventFormatterClassName == null ) {
            logger.log( Level.INFO, "(eventFormatterClassName == null)" );
            throw new EventFormatterNotFoundException( "A formatter was not listed for the event type, " + type + ", in the mapping file " + FORMATTERFACTORY_PROPERTIES + "." );
        }
        try {
            return (EventFormatter)Class.forName( eventFormatterClassName ).newInstance( );
        }
        catch( InstantiationException e ) {
            throw new EventFormatterException( "Unable to instantiate the event formatter class " + eventFormatterClassName + "to format events of type " + type + ".", e );
        }
        catch( IllegalAccessException e ) {
            logger.log( Level.INFO, "Encountered access problems during instantiation of the event formatter class " + eventFormatterClassName + "to format events of type " + type + "." );
            throw new EventFormatterException( "Unable to instantiate the event formatter class " + eventFormatterClassName + "to format events of type " + type + ".", e );
        }
        catch( ClassNotFoundException e ) {
            logger.log( Level.INFO, "The event formatter class " + eventFormatterClassName + " could not be located for loading." );
            throw new EventFormatterException( "Unable to instantiate the event formatter class " + eventFormatterClassName + "to format events of type " + type + ".", e );
        }
    }

    /*
     * Loads the event type-to-formatter mapping defined in the
     * DB2EventFormatter.properties file into a properties object.
     */
    private void initializeProperties( ) throws EventFormatterException {
        synchronized( lock ) {
            if( properties != null ) {
                return;
            }
            properties = new Properties( );

            InputStream in = getClass( ).getResourceAsStream( FORMATTERFACTORY_PROPERTIES );
            if( in == null ) {
                logger.log( Level.INFO, "(in == null)" );
                throw new EventFormatterException( "The input stream to the mapping file " + FORMATTERFACTORY_PROPERTIES + " is null.  The file could not be located for reading." );
            }
            try {
                properties.load( in );
            }
            catch( IOException e ) {
                throw new EventFormatterException( "There was a problem with loading the mapping file " + FORMATTERFACTORY_PROPERTIES + ".", e );
            }
        }
    }
}