// ===================================================================
// (C) Copyright IBM Corp. 2005, 2006 All Rights Reserved.
//                                                                           
// Licensed Materials - Property of IBM
//                                                          
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//              
// You may copy, modify, and distribute this sample program in any
// form without payment to IBM, for any purpose including developing,
// using, marketing or distributing programs that include or are
// derivative works of the sample program.
//                                                                           
// The sample program is provided to you on an "AS IS" basis, without
// warranty of any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES,
// EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Some jurisdictions do not allow for the exclusion or limitation of
// implied warranties, so the above limitations or exclusions may not
// apply to you. IBM shall not be liable for any damages you suffer as
// a result of using, modifying or distributing the sample program or
// its derivatives.
//===================================================================
package com.ibm.wbimonitor.samples.db2emitter;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.sql.DataSource;

import com.ibm.wbimonitor.samples.db2emitter.exception.EventTablePollerException;

/*
 * Polls the database events table for inbound event and archives(purges) the
 * processed events to the log table.
 */
public class EventTablePoller {

    private static Logger logger = Logger.getLogger( "EventTablePoller" );

    private static final int POLL_SIZE_DEFAULT = 100;

    private static String POLL_SQL = "SELECT * FROM EVENTTABLE FOR UPDATE";

    private static String PURGE_PROC = "{CALL LOGPROC(?,?)}";

    private static DataSource dataSource = null;

    /*
     * Poll the database events table for inbound events. Up to
     * POLL_SIZE_DEFAULT (POLL_SIZE_DEFAULT = 100) will be processed per polling
     * period.
     */
    public static Collection poll( ) throws EventTablePollerException {

        try {
            if( dataSource == null )
                dataSource = DB2Helper.getDataSource( );
        }
        catch( NamingException ne ) {
            throw new EventTablePollerException( "Unable to obtain the datasource for the database.", ne );
        }

        Connection connection = null;
        Statement statement = null;

        LinkedList eventsList = new LinkedList( );
        try {
            connection = dataSource.getConnection( );
            statement = connection.createStatement( ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE );
            statement.setMaxRows( POLL_SIZE_DEFAULT );

            ResultSet rs = statement.executeQuery( POLL_SQL );

            while( rs.next( ) ) {
                long sid = rs.getLong( "SID" );

                EventTableBean bean = new EventTableBean( );
                bean.setSid( sid );
                bean.setType( rs.getString( "TYPE" ) );
                bean.setTrigger( (int)rs.getString( "TRIGGER" ).charAt( 0 ) );
                bean.setKeyType( rs.getString( "KEYTYPE" ) );
                bean.setKeyValue( rs.getString( "KEYVALUE" ) );
                bean.setTimestamp( rs.getTimestamp( "CREATETIMESTAMP" ) );
                eventsList.add( bean );
            }
        }
        catch( SQLException se ) {
            try {
                if( connection != null ) {
                    connection.rollback( );
                }
            }
            catch( SQLException se1 ) {
                logger.log( Level.INFO, "Transaction rollback failed.", se1 );
            }
            throw new EventTablePollerException( "Exception occured while retrieving summarized event data from the event table.", se );
        }
        finally {
            try {
                if( statement != null ) {
                    statement.close( );
                }
                if( connection != null ) {
                    connection.close( );
                }
            }
            catch( SQLException se ) {
                logger.log( Level.INFO, "Exception ocurred while attempting to close handles to the statement or connection", se );
                throw new Error( "Exception ocurred while attempting to close handles to the statement or connection", se );
            }
        }

        return eventsList;
    }

    /*
     * Archive the event record in the events table to the logs table. If the
     * boolean argument is true (indicating successful emission of the CBE
     * event), the log record is tagged with an 'S'. Otherwise, if it is false,
     * the event record is flagged with an 'F'.
     */
    public static void purge( EventTableBean bean, boolean failed ) throws EventTablePollerException {

        try {
            if( dataSource == null )
                dataSource = DB2Helper.getDataSource( );
        }
        catch( NamingException ne ) {
            throw new EventTablePollerException( "Unable to obtain the datasource for the database.", ne );
        }

        CallableStatement callableStatement = null;
        Connection connection = null;
        try {
            connection = dataSource.getConnection( );

            String result = ( failed ? "F" : "S" );
            callableStatement = connection.prepareCall( PURGE_PROC );
            callableStatement.setDouble( 1, bean.getSid( ) );
            callableStatement.setString( 2, result );
            callableStatement.executeUpdate( );
        }
        catch( SQLException se ) {
            try {
                if( connection != null ) {
                    connection.rollback( );
                }
            }
            catch( SQLException se1 ) {
                logger.log( Level.INFO, "Transaction rollback failed.", se1 );
            }
            throw new EventTablePollerException( "Exception occured while archiving event data from the event table to the log table.", se );
        }
        finally {
            try {
                if( callableStatement != null ) {
                    callableStatement.close( );
                }
                if( connection != null ) {
                    connection.close( );
                }
            }
            catch( SQLException se ) {
                logger.log( Level.INFO, "Exception ocurred while attempting to close handles to the statement or connection", se );
                throw new Error( "Exception ocurred while attempting to close handles to the statement or connection", se );
            }
        }
    }
}