package com.ibm.wbimonitor.samples.emitterframework.poll;

import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.websphere.scheduler.BeanTaskInfo;
import com.ibm.websphere.scheduler.NotificationException;
import com.ibm.websphere.scheduler.NotificationSinkInvalid;
import com.ibm.websphere.scheduler.Scheduler;
import com.ibm.websphere.scheduler.SchedulerNotAvailableException;
import com.ibm.websphere.scheduler.TaskHandlerHome;
import com.ibm.websphere.scheduler.TaskInfoInvalid;
import com.ibm.websphere.scheduler.TaskInvalid;
import com.ibm.websphere.scheduler.TaskPending;
import com.ibm.websphere.scheduler.TaskStatus;
import com.ibm.websphere.scheduler.UserCalendarInvalid;
import com.ibm.websphere.scheduler.UserCalendarPeriodInvalid;
import com.ibm.websphere.scheduler.UserCalendarSpecifierInvalid;

public class EmitterFWTaskSchedulerUtil {

    private static Logger logger = Logger.getLogger( "EmitterFWTaskSchedulerUtil" );

    private String schedulerJNDI;

    /*
     * The default JNDI name of the application server scheduler that schedules
     * the task that will perform the periodic polling. This default JNDI name
     * is used if no explicit one is configured via an EJB deployment
     * descriptor's schedulerJNDI environment entry (i.e.,
     * "java:comp/env/schedulerJNDI").
     */
    private static final String SCHEDULER_JNDI_DEFAULT = "sched/FolderPoller";

    private String taskHandlerJNDI;

    private String taskInfoName;

    public EmitterFWTaskSchedulerUtil( String taskHandlerJNDI, String taskInfoName ) {
        this.taskHandlerJNDI = taskHandlerJNDI;
        this.taskInfoName = taskInfoName;
    }

    public boolean start( ) {
        logger.entering( "EmitterFWTaskSchedulerUtil", "start" );

        boolean status = false;

        InitialContext ic = null;
        try {
            ic = new InitialContext( );
            schedulerJNDI = (String)ic.lookup( "java:comp/env/schedulerJNDI" );
        }
        catch( NamingException ne ) {
            logger.log( Level.INFO, "Unable to lookup of the environment entry \"schedulerJNDI\".", ne );
        }

        if( ( schedulerJNDI == null ) || ( schedulerJNDI.length( ) == 0 ) ) {
            logger.log( Level.INFO, "The JNDI name for the scheduler has not been set. The default JNDI name, " + SCHEDULER_JNDI_DEFAULT + ", will be used." );
            schedulerJNDI = SCHEDULER_JNDI_DEFAULT;
        }

        Scheduler scheduler = null;
        try {
            if( ic == null )
                ic = new InitialContext( );
            scheduler = (Scheduler)ic.lookup( schedulerJNDI );
        }
        catch( NamingException ne ) {
            logger.log( Level.INFO, "Unable to lookup of the scheduler with the JNDI name " + schedulerJNDI + ".", ne );
            return false;
        }

        if( scheduler == null ) {
            logger.log( Level.INFO, "The scheduler with the JNDI name " + schedulerJNDI + " could not be located (null)." );
            return false;
        }

        try {
            BeanTaskInfo taskInfo = (BeanTaskInfo)scheduler.createTaskInfo( BeanTaskInfo.class );
            Object o = ic.lookup( taskHandlerJNDI );
            TaskHandlerHome home = (TaskHandlerHome)javax.rmi.PortableRemoteObject.narrow( o, TaskHandlerHome.class );
            taskInfo.setName( taskInfoName );
            taskInfo.setTaskHandler( home );
            taskInfo.setAutoPurge( false );
            taskInfo.setStartTime( new java.util.Date( System.currentTimeMillis( ) ) );
            taskInfo.setStartTimeInterval( "20SECONDS" );
            taskInfo.setNumberOfRepeats( -1 );
            taskInfo.setRepeatInterval( "10SECONDS" );
            TaskStatus ts = scheduler.create( taskInfo );
            System.out.println( "Created task with id: " + ts.getTaskId( ) );

            status = true;
        }
        catch( TaskInfoInvalid tii ) {
            logger.log( Level.INFO, "The scheduler was unable to create a TaskInfo object.", tii );
        }
        catch( NamingException ne ) {
            logger.log( Level.INFO, "Unable to lookup of the task handler with the JNDI name " + taskHandlerJNDI + ".", ne );
        }
        catch( RemoteException re ) {
            logger.log( Level.INFO, "Error adding the TaskHandler information to the TaskInfo object.", re );
        }
        catch( UserCalendarSpecifierInvalid ucsi ) {
            logger.log( Level.INFO, "", ucsi );
        }
        catch( UserCalendarInvalid uci ) {
            logger.log( Level.INFO, "", uci );
        }
        catch( UserCalendarPeriodInvalid icpi ) {
            logger.log( Level.INFO, "", icpi );
        }
        catch( TaskInvalid ti ) {
            logger.log( Level.INFO, "", ti );
        }
        catch( SchedulerNotAvailableException snae ) {
            logger.log( Level.INFO, "The scheduler service is not available.", snae );
        }
        catch( NotificationSinkInvalid nsi ) {
            logger.log( Level.INFO, "", nsi );
        }
        catch( NotificationException ne ) {
            logger.log( Level.INFO, "", ne );
        }

        logger.exiting( "EmitterFWTaskSchedulerUtil", "start" );

        return status;
    }

    public void stop( ) {
        logger.entering( "EmitterFWTaskSchedulerUtil", "stop" );

        try {
            InitialContext ic = new InitialContext( );
            Scheduler scheduler = (Scheduler)ic.lookup( schedulerJNDI );

            Iterator iterator = scheduler.findTaskStatusByName( taskInfoName );
            while( iterator.hasNext( ) ) {
                TaskStatus ts = (TaskStatus)iterator.next( );
                String taskId = ts.getTaskId( );
                scheduler.cancel( taskId, true );

                logger.log( Level.INFO, "Task with ID: " + ts.getTaskId( ) + " has been cancelled." );
            }
        }
        catch( NamingException ne ) {
            logger.log( Level.INFO, "Unable to lookup of the scheduler with the JNDI name " + schedulerJNDI + ".", ne );
        }
        catch( SchedulerNotAvailableException snae ) {
            logger.log( Level.INFO, "The scheduler service is not available.", snae );
        }
        catch( TaskInvalid ti ) {
            logger.log( Level.INFO, "", ti );
        }
        catch( TaskPending tp ) {
            logger.log( Level.INFO, "", tp );
        }
        catch( NotificationException ne ) {
            logger.log( Level.INFO, "", ne );
        }

        logger.exiting( "EmitterFWTaskSchedulerUtil", "stop" );

    }
}
